package fiftyShadesOfMultiThreading.parallelSummers;
import fiftyShadesOfMultiThreading.summers.SummerCallable;
import fiftyShadesOfMultiThreading.worker.WorkPartitioner;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ExecutorService;  
import java.util.concurrent.Executors;
import java.util.concurrent.Future;

import fiftyShadesOfMultiThreading.worker.WorkPartitioner.Part;


public class ParallelSummer3 {
	
	public static long sum(int[] data, int threadCount) {
		
		//creating a pool of #threadCount threads  
		ExecutorService executor = Executors.newFixedThreadPool(threadCount);
		
		List<SummerCallable> summers =  new ArrayList<SummerCallable>();
		List<Part> parts =  WorkPartitioner.partitions(data.length, threadCount);
		
		for (Part part : parts) {
			summers.add(new SummerCallable(data,part));
		}
		
		List<Future<Long>> results;
		try {
			results = executor.invokeAll(summers);
		} catch (InterruptedException e) {
			System.err.println("Cannot invoke the threads.");
			return -1;
		}
		
		executor.shutdown();
	    
	    long sum = 0;
	    for (Future<Long> future : results) {
			try {
				sum += future.get();
			} catch (InterruptedException | ExecutionException e) {
				System.err.println("Cannot get the results from threads.");
				return -2;
			}
		}
	    return sum;
	}

}
